// Qt headers
#include <QApplication>
#include <QLineEdit>

// gemc headers
#include "gemc_MainGui.h"
#include "string_utilities.h"

// All icon images must be 96x84
#include "images/run_control_xpm.h"
#include "images/camera_control_xpm.h"
#include "images/detector_inspector_xpm.h"
#include "images/infos_xpm.h"
#include "images/g4dialog_xpm.h"
#include "images/analog_signal_xpm.h"
#include "images/physics_xpm.h"

// C++ headers
#include <iostream>
#include <string>
using namespace std;

gemcMainWidget::gemcMainWidget(goptions *Opts, G4RunManager *rm,  map<string, sensitiveDetector*> SDM, map<string, detector> *hMap, map<string, G4Material*> matMap)
{
	gemcOpt    = Opts;
	RM = rm;

	// revisit this, why we need it?
	double qt_mode = gemcOpt->optMap["USE_GUI"].arg ;
	
	contentsWidget = new QListWidget;
	contentsWidget->setViewMode(QListView::IconMode);
	
	// icon size
	contentsWidget->setIconSize(QSize(74, 64));
	contentsWidget->setMovement(QListView::Static);

	// icon container sizes
	contentsWidget->setMinimumWidth(125);
	contentsWidget->setMaximumWidth(125);
	contentsWidget->setMinimumHeight(710);
	contentsWidget->setMaximumHeight(710);
	
	// makes all icon the same size
	contentsWidget->setUniformItemSizes(1);
	
	// activate tracking (for mouse over effect)
	contentsWidget->setMouseTracking(1);
	
	contentsWidget->setSpacing(8);

	// Content:
	// Run Control
	// Camera Control
	// Infos
	// G4 Dialog
	// Oscilloscope Signal
	// Physics List Control
  
	pagesWidget = new QStackedWidget;
	pagesWidget->addWidget(new run_control           (this, gemcOpt));
	pagesWidget->addWidget(new camera_control        (this, gemcOpt));
	pagesWidget->addWidget(new detector_tree         (this, *gemcOpt, RM, hMap,  matMap));
	pagesWidget->addWidget(new infos                 (this, gemcOpt));
	pagesWidget->addWidget(new g4dialog              (this, gemcOpt));
	pagesWidget->addWidget(gsig = new gsignal        (this, gemcOpt, SDM));
	pagesWidget->setMinimumWidth(650);
	pagesWidget->setMaximumWidth(650);
	pagesWidget->setMinimumHeight(750);
	pagesWidget->setMaximumHeight(750);
	QPushButton *closeButton = new QPushButton(tr("Exit"));
	
	createIcons();
	contentsWidget->setCurrentRow(0);
	
	// revisit this, why we need it?
	if(qt_mode > 1)
		contentsWidget->setCurrentRow(1);
	
	connect(closeButton, SIGNAL(clicked()), this, SLOT(gemc_quit()));
	
	QHBoxLayout *horizontalLayout = new QHBoxLayout;
	horizontalLayout->addWidget(contentsWidget);
	horizontalLayout->addWidget(pagesWidget, 1);
	
	QHBoxLayout *buttonsLayout = new QHBoxLayout;
	buttonsLayout->addStretch(1);
	buttonsLayout->addWidget(closeButton);
	
	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addLayout(horizontalLayout);
	mainLayout->addLayout(buttonsLayout);
	setLayout(mainLayout);
	
	setWindowTitle(tr("Config Dialog"));

	return;
}



void gemcMainWidget::changePage(QListWidgetItem *current, QListWidgetItem *previous)
{
	if (!current)
		current = previous;
	
	// manually setting the content index
	if(contentsWidget->row(current) == 0) pagesWidget->setCurrentIndex(0);
	if(contentsWidget->row(current) == 1) pagesWidget->setCurrentIndex(1);
	if(contentsWidget->row(current) == 2) pagesWidget->setCurrentIndex(2);
	if(contentsWidget->row(current) == 3) pagesWidget->setCurrentIndex(3);
	if(contentsWidget->row(current) == 4) pagesWidget->setCurrentIndex(4);
	if(contentsWidget->row(current) == 5)
	{
		pagesWidget->setCurrentIndex(5);
		gsig->CreateSDetsTree();
	}
	if(contentsWidget->row(current) == 6) pagesWidget->setCurrentIndex(6);
}

void gemcMainWidget::createIcons()
{
	i_default = QColor( 124,  164, 210);
	i_hovered = QColor( 250,  140, 140);
	
	QListWidgetItem *rcontrol = new QListWidgetItem(contentsWidget);
	rcontrol->setIcon(QPixmap(run_control_xpm));
	rcontrol->setText(tr("Run Control"));
	rcontrol->setBackground(QBrush(i_default));
	rcontrol->setTextAlignment(Qt::AlignHCenter);
	rcontrol->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled);

	QListWidgetItem *ccontrol = new QListWidgetItem(contentsWidget);
	ccontrol->setIcon(QPixmap(camera_control_xpm));
	ccontrol->setText(tr("Camera"));
	ccontrol->setBackground(QBrush(i_default));
	ccontrol->setTextAlignment(Qt::AlignHCenter);
	ccontrol->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	
	QListWidgetItem *dinspector = new QListWidgetItem(contentsWidget);
	dinspector->setIcon(QPixmap(detector_inspector_xpm));
	dinspector->setText(tr("Detector"));
	dinspector->setBackground(QBrush(i_default));
	dinspector->setTextAlignment(Qt::AlignHCenter);
	dinspector->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);

	QListWidgetItem *infos = new QListWidgetItem(contentsWidget);
	infos->setIcon(QPixmap(infos_xpm));
	infos->setText(tr("Infos"));
	infos->setBackground(QBrush(i_default));
	infos->setTextAlignment(Qt::AlignHCenter);
	infos->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  
	QListWidgetItem *g4dialog = new QListWidgetItem(contentsWidget);
	g4dialog->setIcon(QPixmap(g4dialog_xpm));
	g4dialog->setText(tr("G4Dialog"));
	g4dialog->setBackground(QBrush(i_default));
	g4dialog->setTextAlignment(Qt::AlignHCenter);
	g4dialog->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
  
	QListWidgetItem *gsignal = new QListWidgetItem(contentsWidget);
	gsignal->setIcon(QPixmap(analog_signal_xpm));
	gsignal->setText(tr("Signals"));
	gsignal->setBackground(QBrush(i_default));
	gsignal->setTextAlignment(Qt::AlignHCenter);
	gsignal->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	
	QListWidgetItem *gphysics = new QListWidgetItem(contentsWidget);
	gphysics->setIcon(QPixmap(physics_xpm));
	gphysics->setText(tr("Physics"));
	gphysics->setBackground(QBrush(i_default));
	gphysics->setTextAlignment(Qt::AlignHCenter);
	gphysics->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	
	connect(contentsWidget,
					SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem*)),
          this, SLOT(changePage(QListWidgetItem *, QListWidgetItem*)));
	
	connect(contentsWidget,	SIGNAL(itemEntered(QListWidgetItem *)), this, SLOT(change_background(QListWidgetItem *)) );
	
}

void gemcMainWidget::change_background(QListWidgetItem* item)
{
	for(int i=0; i<contentsWidget->count(); i++)
		contentsWidget->item(i)->setBackground(QBrush(i_default));
	
	item->setBackground(QBrush(i_hovered));

}


gemcMainWidget::~gemcMainWidget()
{
	string hd_msg = gemcOpt->optMap["LOG_MSG"].args;


	cout << endl;
}





