#ifndef momControls_H
#define momControls_H 1

// Qt headers
#include <QSlider>
#include <QLabel>
#include <QtGui>

// gemc headers
#include "options.h"


class momControls : public QWidget
{
	Q_OBJECT
	
	public:
		
		momControls(goptions *Opts, QWidget *parent, string type);
	
	public:
		QComboBox *beam_particle;
		QSlider *momentum_slider;
		QSlider *rmomentum_slider;
		QSlider *theta_slider;
		QSlider *rtheta_slider;
		QSlider *phi_slider;
		QSlider *rphi_slider;
	
		// this will return a value rounded up
		// if the returned value is greater than "max"
		// a finer definition of "rounded up" is used.
		double get_max_ten_from_value(double v, double max)
		{
			
			double next_magnitude                      = 1;
			while (next_magnitude < v) next_magnitude *= 10;
			
			double last_magnitude = 0;
			double fine_grid = 50;
			
			while(last_magnitude < v)
			{
				last_magnitude += next_magnitude / fine_grid;
				if(last_magnitude > max)
				{
					last_magnitude = 0;
					fine_grid	*= 10;
				}
			}
			
			return last_magnitude;
		}
	
		// momentum
		double mom_sliders_nticks;
		double mom_slider_min;
		double mom_slider_max;
		// min value for slider is fixed 0
		double rmom_slider_max;
	
		double theta_slider_range;
		double theta_slider_scale;
	
		double theta_slider_min;
		double theta_slider_max;
	
		double phi_slider_range;
		double phi_slider_scale;
		
		double get_rel_pos()           {return (momentum_slider->sliderPosition())/(mom_sliders_nticks-1);}
		double get_dmom()              {return (mom_slider_max - mom_slider_min);}
		double get_mom_from_slider()   {return get_rel_pos()*get_dmom() + mom_slider_min;}
		double get_rmom_from_slider()  {return rmomentum_slider->sliderPosition()/(mom_sliders_nticks-1)*rmom_slider_max;}
		
		double get_theta_from_slider() {return  theta_slider->sliderPosition()*theta_slider_scale;}
		double get_rtheta_from_slider(){return rtheta_slider->sliderPosition()*theta_slider_scale;}
		
		double get_phi_from_slider()   {return  phi_slider->sliderPosition()*phi_slider_scale;}
		double get_rphi_from_slider()  {return rphi_slider->sliderPosition()*phi_slider_scale/2.0;}
		
		QLabel *momentum_ro;
		QLabel *theta_ro;
		QLabel *phi_ro;
		
		QLabel *rmomentum_ro;
		QLabel *rtheta_ro;
		QLabel *rphi_ro;
	
		void set_momentum_ro();
		void set_theta_ro();
		void set_phi_ro();
		
		void set_rmomentum_ro();
		void set_rtheta_ro();
		void set_rphi_ro();
		
		QGroupBox *momentumGroup;
		QGroupBox *mroGroup;
	
		QPushButton *momPlus;
	
	private slots:
	
		void increaseMomZoon()
		{
			double mom = get_mom_from_slider();
			
			// max is between mom and max
			mom_slider_max = get_max_ten_from_value( mom, (mom + mom_slider_max)/2.0);
			// min is between min and mom
			mom_slider_min = get_max_ten_from_value( (mom + mom_slider_min)/2.0 , mom);
			
			// setting slider position so momentum doesn't change
			// the 0.5 is added because of the (int) truncation
			momentum_slider->setSliderPosition((int) ((mom-mom_slider_min)/get_dmom()*(mom_sliders_nticks-1) + 0.5 ) );
		
		
		
			double rmom = get_rmom_from_slider();
			rmom_slider_max = 2*mom_slider_max;
			rmomentum_slider->setSliderPosition((int) (rmom/rmom_slider_max*(mom_sliders_nticks-1) + 0.5 ) );

		}
	
	
	
		void decreaseMomZoon()
		{
			double mom = get_mom_from_slider();
			
			// max is between max and 10 max
			mom_slider_max = get_max_ten_from_value( 2*mom_slider_max , 4*mom_slider_max);
			// min is between twice min and min
			mom_slider_min = get_max_ten_from_value( 2*mom_slider_min - mom , mom_slider_min);
			
			
			// setting slider position so momentum doesn't change
			// the 0.5 is added because of the (int) truncation
			momentum_slider->setSliderPosition((int) ((mom-mom_slider_min)/get_dmom()*(mom_sliders_nticks-1) + 0.5 ) );

			double rmom = get_rmom_from_slider();
			rmom_slider_max = 2*mom_slider_max;
			rmomentum_slider->setSliderPosition((int) (rmom/rmom_slider_max*(mom_sliders_nticks-1) + 0.5 ) );
			
		}
	
};


#endif






