// gemc headers
#include "vtxControls.h"
#include "string_utilities.h"



vtxControls::vtxControls(goptions *Opts, QWidget *parent, string type)
{	
	// vector of string - filled from the various option
	vector<string> values;
	string units;

	
	// Getting vertex from option value
	if(type == "Primary") values    = get_info(Opts->optMap["BEAM_V"].args);
	if(type == "Lumi1")   values    = get_info(Opts->optMap["LUMI_V"].args);
	if(type == "Lumi2")   values    = get_info(Opts->optMap["LUMI2_V"].args);
	units = TrimSpaces(values[3]);
	double cvx = get_number(values[0] + "*" + units);
	double cvy = get_number(values[1] + "*" + units);
	double cvz = get_number(values[2] + "*" + units);
	
	// Getting vertex spread from option value
	if(type == "Primary") values    = get_info(Opts->optMap["SPREAD_V"].args);
	if(type == "Lumi1")   values    = get_info(Opts->optMap["LUMI_SPREAD_V"].args);
	if(type == "Lumi2")   values    = get_info(Opts->optMap["LUMI2_SPREAD_V"].args);
	units = TrimSpaces(values[2]);
	double cdvr = get_number(values[0] + "*" + units);
	double cdvz = get_number(values[1] + "*" + units);
	
	
	// VX and RV
	vx_slider_range = 10000;
	vx_slider_scale = 1;
	
	vx_slider = new QSlider(Qt::Horizontal);
	vx_slider->setRange(-vx_slider_range, vx_slider_range);
	vx_slider->setSliderPosition((int) (cvx/vx_slider_scale));
	
	rv_slider_range = 100;
	rv_slider_scale = 1;

	rv_slider = new QSlider(Qt::Horizontal);
	rv_slider->setRange(0, rv_slider_range);
	rv_slider->setSliderPosition((int) (cdvr/rv_slider_scale));
	
	QHBoxLayout *vxLayout = new QHBoxLayout;
	vxLayout->addWidget(new QLabel(tr("vx:")));
	vxLayout->addWidget(vx_slider);
	vxLayout->addSpacing(30);
	vxLayout->addWidget(new QLabel(tr("radius:")));
	vxLayout->addWidget(rv_slider);
	connect ( vx_slider , SIGNAL( valueChanged(int) ), parent, SLOT( set_vertex_values() ) );
	connect ( rv_slider , SIGNAL( valueChanged(int) ), parent, SLOT( set_vertex_values() ) );
	
	
	// VY and RVZ
	vy_slider_range = 10000;
	vy_slider_scale = 1;
	
	vy_slider = new QSlider(Qt::Horizontal);
	vy_slider->setRange(-vy_slider_range, vy_slider_range);
	vy_slider->setSliderPosition((int) (cvy/vy_slider_scale));
	
	rvz_slider_range = 100;
	rvz_slider_scale = 1;
 
	rvz_slider = new QSlider(Qt::Horizontal);
	rvz_slider->setRange(0, rvz_slider_range);
	rvz_slider->setSliderPosition((int) (cdvz/rvz_slider_scale));
	
	QHBoxLayout *vyLayout = new QHBoxLayout;
	vyLayout->addWidget(new QLabel(tr("vy:")));
	vyLayout->addWidget(vy_slider);
	vyLayout->addSpacing(46);
	vyLayout->addWidget(new QLabel(tr("dvz:")));
	vyLayout->addWidget(rvz_slider);
	connect ( vy_slider ,  SIGNAL( valueChanged(int) ), parent, SLOT( set_vertex_values() ) );
	connect ( rvz_slider , SIGNAL( valueChanged(int) ), parent, SLOT( set_vertex_values() ) );
	
	
	
	// VZ
	vz_slider_range = 10000;
	vz_slider_scale = 1;
	
	vz_slider = new QSlider(Qt::Horizontal);
	vz_slider->setRange(-vz_slider_range, vz_slider_range);
	vz_slider->setTickInterval(1);
	vz_slider->setSliderPosition((int) (cvz/vz_slider_scale));
	
	QHBoxLayout *vzLayout = new QHBoxLayout;
	vzLayout->addWidget(new QLabel(tr("vz:")));
	vzLayout->addWidget(vz_slider);
	vzLayout->addSpacing(305);
	connect ( vz_slider ,  SIGNAL( valueChanged(int) ), parent, SLOT( set_vertex_values() ) );
	

	// Vertex Group
	QHBoxLayout *explanaLayout2 = new QHBoxLayout;
	explanaLayout2->addWidget(new QLabel(tr("")));
	explanaLayout2->addWidget(new QLabel(tr("Value")));
	explanaLayout2->addWidget(new QLabel(tr("Dispersion")));
	

	
	vertexGroup = new QGroupBox(tr("Vertex"));
	vertexGroup->setMaximumHeight(148);

	QVBoxLayout *vLayout = new QVBoxLayout;
	vLayout->addLayout(explanaLayout2);
	vLayout->addLayout(vxLayout);
	vLayout->addLayout(vyLayout);
	vLayout->addLayout(vzLayout);
	vertexGroup->setLayout(vLayout);	
	vertexGroup->setStyleSheet("   * { background-color: rgb(215, 215, 220);} QLabel {background-color: transparent; }");


	// Vertex read out Group
	vroGroup = new QGroupBox(tr("Vertex Values"));
	vroGroup->setMaximumHeight(125);

	QHBoxLayout *vxy_roLayout = new QHBoxLayout;
	vxy_roLayout->addWidget(new QLabel(tr("(x,y,z):")));
	vxyz_ro = new QLabel(tr("(0, 0, 0) mm"));
	vxy_roLayout->addWidget(vxyz_ro);
	
	QHBoxLayout *vdr_roLayout = new QHBoxLayout;
	vdr_roLayout->addWidget(new QLabel(tr("radius:")));
	vdr_ro = new QLabel(tr("0 mm"));
	vdr_roLayout->addWidget(vdr_ro);
	
	QHBoxLayout *vdz_roLayout = new QHBoxLayout;
	vdz_roLayout->addWidget(new QLabel(tr("delta z:")));
	vdz_ro = new QLabel(tr("0 mm"));
	vdz_roLayout->addWidget(vdz_ro);
	
	QVBoxLayout *vroLayout = new QVBoxLayout;
	vroLayout->addLayout(vxy_roLayout);
	vroLayout->addLayout(vdr_roLayout);
	vroLayout->addLayout(vdz_roLayout);
	vroGroup->setLayout(vroLayout);
	vroGroup->setStyleSheet("      * { background-color: rgb(215, 215, 220);} QLabel {background-color: transparent; }");


}

void vtxControls::set_vxyz_ro()
{
	double vx  = get_vx_from_slider();
	double vy  = get_vy_from_slider();
	double vz  = get_vz_from_slider();
	
	QString vro = "(";
	vro.append(stringify(vx).c_str());
	vro.append(", ");
	vro.append(stringify(vy).c_str());
	vro.append(", ");
	vro.append(stringify(vz).c_str());
	vro.append(") mm");
	vxyz_ro->setText(vro);
}



void vtxControls::set_vdr_ro()
{
	double dvr  = get_rv_from_slider();
	QString dvro(stringify(dvr).c_str());
	dvro.append(" mm");
	vdr_ro->setText(dvro);
}



void vtxControls::set_vdz_ro()
{
	double dvz  = get_rvz_from_slider();
	QString dvzro(stringify(dvz).c_str());
	dvzro.append(" mm");
	vdz_ro->setText(dvzro);
}




